#!/usr/bin/python
# -*- coding: utf-8 -*-

# Copyright (C) 2007   Lucas Bonnet for Bearstech <lbonnet@bearstech.com>

# Licence : GPL V2 or V3

import pygtk
pygtk.require('2.0')
import gtk
import gobject
import os.path
import os  # for spawn
import signal
import time # for sleep
import commands # for getoutput


#config
nmea_file = "/tmp/nmeaNP"
trace = True
trace_file = "/media/card/trace.log"
gllin_path = "/home/root/gllin/gllin"

class GpsRead:

    def delete_event(self, widget, event, data=None):
        # Change FALSE to TRUE and the main window will not be destroyed
        # with a "delete_event".
        self.kill_gps()
        return False

    def destroy(self, widget, data=None):
        self.kill_gps()
        gtk.main_quit()

    def parse_gga(self):
        # fetch all the data at once
        (format, utc, latitude, northsouth,
         longitude, eastwest, quality,
         number_of_satellites_in_use,
         horizontal_dilution, altitude,
         above_sea_unit, geoidal_separation,
         geoidal_separation_unit, data_age,
         diff_ref_stationID) = self.splitted_line

        # build the timestamp in a human-readable form
        timestamp = utc[0:2] + ":" + utc[2:4] + ":" + utc[4:6]
        self.fld_tstamp.set_text(timestamp)

        self.fld_alt.set_text(altitude)
        self.fld_sat.set_text(number_of_satellites_in_use.lstrip('0'))

        # compute coordinates (lat/long) and display them in a human-readable form
        if latitude:
            coord_pretty = self.parse_coord(latitude, longitude, northsouth, eastwest)
            self.fld_coord.set_text(coord_pretty)
        # end if latitude
    #end parse_gga

    def parse_coord(self,latitude, longitude, northsouth, eastwest):
        latitude_f  = float(latitude)
        longitude_f = float(longitude)
        
        latitude_degrees = int(latitude_f/100)
        latitude_minutes = latitude_f - latitude_degrees*100
        latitude_seconds = float( "0." + latitude.split(".")[1]) * 60
        
        longitude_degrees = int(longitude_f/100)
        longitude_minutes = longitude_f - longitude_degrees*100
        longitude_seconds = float( "0." + longitude.split(".")[1]) * 60
        
        latitude = latitude_degrees + (latitude_minutes/60)
        longitude = longitude_degrees + (longitude_minutes/60)

        # human-friendly coordinates
        return "%d° %d' %.3f\" %s\n%d° %d' %.3f\" %s" % \
               (latitude_degrees,latitude_minutes,latitude_seconds,northsouth,
                longitude_degrees,longitude_minutes,longitude_seconds,eastwest)
    # end parse_coord

    def read_line(self,source, condition):
        line = self.file.readline()
        self.splitted_line = line.split(",")

        # did we get a GPGGGA line ?
        if self.splitted_line[0] == "$GPGGA":
            self.parse_gga()
            if trace:
                self.trace_file.write(line)
        
        # return False to deactivate callback, True to let it run
        return True
    # end read_line

    def spawn_gps(self, widget, data=None):
        print "starting GPS..."
        self.pid = os.spawnl(os.P_NOWAIT,gllin_path)
        self.lbl_status.set_text("Starting...")
        # wait 4 seconds to let gllin boot properly, and then start parsing data
        source_id = gobject.timeout_add(4000, self.init_gps_reader,self)
    # end spawn_gps

    def kill_gps(self,widget=None,data=None):
        if self.pid:
            print "killing GPS..."
            os.kill(self.pid,signal.SIGKILL)
            # remove temp file
            os.unlink(nmea_file)
            self.pid = 0

    def gps_is_off(self):
        self.lbl_status.set_text("No GPS found, please start it")
        self.btn_start.set_sensitive(1)

    def gps_is_on(self):
        self.lbl_status.set_text("")
        self.btn_start.set_sensitive(0)

    def init_gps_reader(self, stuff=None):
        if os.path.exists(nmea_file):
            # try to find if someone is using that file
            fuser_output = commands.getoutput("fuser " + nmea_file)
            # no one? then gllin isn't running, do not bother trying to open the pipe
            if fuser_output == "":
                self.gps_is_off()
            else: # there is a process accessing that file, it's likely to be gllin, so open it
                self.file = open(nmea_file,'r')
                self.gps_source = gobject.io_add_watch(self.file, gobject.IO_IN, self.read_line)
                self.gps_is_on()
        else: # no nmea_file
            self.gps_is_off()

        # disable future runs, it's only necessary to run it once
        return False
    # end init_gps_reader
    
    def __init__(self):
        
        # create a new window
        self.window = gtk.Window(gtk.WINDOW_TOPLEVEL)
        self.window.set_title("GPS Reader")

        # connect events
        self.window.connect("delete_event", self.delete_event)
        self.window.connect("destroy", self.destroy)
        
        # Sets the border width of the window.
        self.window.set_border_width(10)

        # our big VerticalBox where all widgets will be placed
        self.vbox = gtk.VBox(homogeneous=0, spacing=5)
        
        # timestamp
        hbox = gtk.HBox(homogeneous=1,spacing=5)
        self.lbl_tstamp = gtk.Label("Time of fix:")
        hbox.pack_start(self.lbl_tstamp)
        self.fld_tstamp = gtk.Label("")
        hbox.pack_start(self.fld_tstamp)
        self.vbox.pack_start(hbox)

        # satellites count
        hbox = gtk.HBox(homogeneous=1,spacing=5)
        self.lbl_sat = gtk.Label("Satellites:")
        hbox.pack_start(self.lbl_sat)
        self.fld_sat = gtk.Label("")
        hbox.pack_start(self.fld_sat)
        self.vbox.pack_start(hbox)

        # coordinates
        hbox = gtk.HBox(homogeneous=1,spacing=5)
        self.lbl_coord = gtk.Label("Coordinates:")
        hbox.pack_start(self.lbl_coord)
        self.fld_coord = gtk.Label("")
        hbox.pack_start(self.fld_coord)
        self.vbox.pack_start(hbox)

        # altitude
        hbox = gtk.HBox(homogeneous=1,spacing=5)
        self.lbl_alt = gtk.Label("Altitude:")
        hbox.pack_start(self.lbl_alt)
        self.fld_alt = gtk.Label("")
        hbox.pack_start(self.fld_alt)
        self.vbox.pack_start(hbox)

        # status line
        self.lbl_status = gtk.Label("")
        self.vbox.pack_start(self.lbl_status)

        # control buttons
        self.bot_hbox = gtk.HBox(homogeneous=1, spacing=5)
        self.btn_start = gtk.Button("Start GPS")
        self.btn_start.connect("clicked", self.spawn_gps)
        self.bot_hbox.pack_start(self.btn_start)
        self.btn_start.set_sensitive(0)
        #self.btn_stop  = gtk.Button("Stop GPS")
        #self.btn_stop.connect( "clicked", self.kill_gps)
        #self.bot_hbox.pack_start(self.btn_stop)
        #self.btn_stop.set_sensitive(0)
        self.btn_quit  = gtk.Button("Quit")
        self.btn_quit.connect_object("clicked", gtk.Widget.destroy, self.window)
        self.bot_hbox.pack_end(self.btn_quit)
        
        self.vbox.pack_end(self.bot_hbox)

        # put the VBox in the window
        self.window.add(self.vbox)

        # show all stuff
        self.window.show_all()

        # prepare GPS reader callback if file already here
        self.init_gps_reader()
        
        if trace:
            self.trace_file = open(trace_file,'w+')

        self.pid = 0
    # end __init__

    def main(self):
        gtk.main()

# If the program is run directly or passed as an argument to the python
# interpreter then create a GpsRead instance and show it
if __name__ == "__main__":
    reader = GpsRead()
    reader.main()
